#!/bin/sh

function usage() {
    echo "Usage: $0 [-a \$path] [-m]"
    exit 1
}

function kill_process() {
    if [ -z == "$1" ]; then
        echo "error. input process argument. check script file."
        exit 1
    fi

    SHELL_NAME=`basename $0`
    PS_TYPE="normal"
    PS_PATH=`which ps`
    REAL_PATH=`readlink $PS_PATH`
    if [ "busybox" == "$REAL_PATH" ]; then
        PS_TYPE="busybox"
    fi

    if [ "normal" == "$PS_TYPE" ]; then
        AGENT=`ps -ef | grep -v grep | grep -v $SHELL_NAME | awk '{print $2,$8}' |grep $1`
    else
        AGENT=`ps -ef | grep -v grep | grep -v $SHELL_NAME | awk '{print $1,$4}' |grep $1`
        if [ -z "$AGENT" ]
        then
            echo `date`
            echo "$1 process is not running."
        else
            echo "kill $1 process PID $AGENT"
            PID=$(echo $AGENT | cut -f1 -d ' ')
            echo "kill pid $PID"
            kill -10 $PID
        fi
    fi
}

function make_sysdir() {
    mkdir -p /etc/rsupport/rvbox/agent
    mkdir -p /var/spool/cron/crontabs

    mkdir -p /var/lock

    mkdir -p /mnt/emmc/log/rsupport
    mkdir -p /var/log
    cd /var/log
    if [ -d /var/log/rsupport ] && [ ! -L /var/log/rsupport ]; then
        mv /var/log/rsupport /var/log/rsupport_legacy
    fi

    if [ ! -L /var/log/rsupport ]; then
        ln -s /mnt/emmc/log/rsupport rsupport
    fi
}

function make_conf() {
    if [ -e /etc/rsupport/rvbox/agent/agent.conf ]; then 
       mv /etc/rsupport/rvbox/agent/agent.conf /etc/rsupport/rvbox/agent/agent.confgQAN5m36
    fi 
    mv /opt/rsupport/rvbox/agent/1.2.0.01/agent.conf /etc/rsupport/rvbox/agent

    if [ -e /etc/rsupport/rvbox/agent/control.conf ]; then 
       mv /etc/rsupport/rvbox/agent/control.conf /etc/rsupport/rvbox/agent/control.confgQAN5m36
    fi 
    mv /opt/rsupport/rvbox/agent/1.2.0.01/control.conf /etc/rsupport/rvbox/agent

    if [ -e /etc/init.d/app.sh ]; then 
       mv /etc/init.d/app.sh /etc/init.d/app.shgQAN5m36
    fi 
    mv /opt/rsupport/rvbox/agent/1.2.0.01/app.sh /etc/init.d/app.sh
    chmod 755 /etc/init.d/app.sh

}
function make_etc() {
    if [ -e /opt/rsupport/rvbox/var/tmp/liveview_failed.jpg ]; then 
       mv /opt/rsupport/rvbox/var/tmp/liveview_failed.jpg /opt/rsupport/rvbox/var/tmp/liveview_failed.jpggQAN5m36
    fi 
    mv /opt/rsupport/rvbox/agent/1.2.0.01/liveview_failed.jpg /opt/rsupport/rvbox/var/tmp

    if [ -e /opt/rsupport/rvbox/var/tmp/liveview_rec.jpg ]; then 
       mv /opt/rsupport/rvbox/var/tmp/liveview_rec.jpg /opt/rsupport/rvbox/var/tmp/liveview_rec.jpggQAN5m36
    fi 
    mv /opt/rsupport/rvbox/agent/1.2.0.01/liveview_rec.jpg /opt/rsupport/rvbox/var/tmp

}

# START
ARCHIVE_PATH=""
INSTALL_TYPE=""

while getopts "a:mh" option
do
    case $option in
    a)
        INSTALL_TYPE="auto"
        ARCHIVE_PATH="$OPTARG"
        ;;
    m)
        INSTALL_TYPE="manual"
        ARCHIVE_PATH=`pwd`
        ;;
    h) usage ;;
    ?) usage ;;
    esac
done

if [ -z $INSTALL_TYPE ] || [ -z $ARCHIVE_PATH ]; then
    echo "no args..."
    usage
fi

if [ ! -d $ARCHIVE_PATH ]; then
    echo "invalid path.."
    exit 1
fi

CHECK_FILE_LIST="agent controller lib/libcrypto.so lib/libspeexdsp.so.1 lib/libspeexdsp.so.1.5.0 lib/librsnet.so lib/libcpr.so lib/libcurl.so lib/libjpeg.so.9 lib/librsnet.so.2.0.2.13 lib/libvideo_system.so lib/libjpeg.so.9.5.0 lib/libspeex.so lib/libproperty.so lib/libtinyxml2.so.7 lib/librspxauth.so lib/libtinyxml2.so.7.0.1 lib/libtinyxml2.so lib/liblispace_network.so lib/librvmedia.so lib/libspeex.so.1.5.1 lib/libjpeg.so lib/libsns_lt6911uxc.so lib/libspeexdsp.so lib/libspeex.so.1 lib/libssl.so lib/libxenc.so lib/liblispace.so djpeg recoder capture tmpworker updater curl "

UPDATE_ROLLBACK="false"
VERSION=1.2.0.01
INSTALL_DIR=/opt/rsupport/rvbox/agent/1.2.0.01
PACKAGE_FILE_NAME=RemoteViewBox-agent-1.2.0.01.tar.gz
cd $ARCHIVE_PATH
echo "1. Installing RemoteViewBOX-agent at /opt/rsupport/rvbox/agent/1.2.0.01"

if [ -d /opt/rsupport/rvbox/agent/1.2.0.01 ]; then  # if pacakage version is same
    mv /opt/rsupport/rvbox/agent/1.2.0.01 /opt/rsupport/rvbox/agent/gQAN5m36
fi
mkdir -p /opt/rsupport/rvbox/agent/1.2.0.01

echo "2. Extract compressed files"
if [ "auto" == "$INSTALL_TYPE" ]
then
    if [ -e .information-agent.gz ]; then
        rm .information-agent.gz
    fi

    if [ -e RemoteViewBox-agent-1.2.0.01.tar.gz ]; then
        rm RemoteViewBox-agent-1.2.0.01.tar.gz
    fi
    cp -a . /opt/rsupport/rvbox/agent/1.2.0.01
else
    tar zxf $ARCHIVE_PATH/RemoteViewBox-agent-1.2.0.01.tar.gz -C /opt/rsupport/rvbox/agent/1.2.0.01
    gzip -d -f .information-agent.gz
    cp .information-agent /opt/rsupport/rvbox/agent/1.2.0.01
    gzip .information-agent
fi

echo "3. make sysdir"
make_sysdir

echo "4. make conf files"
make_conf

echo "5. make etc files"
make_etc

echo "6. Stop cron schedule"
crontab -r

echo "7. Kill agent proess"
kill_process agent

echo "8. Check install files"
for CHECK_FILE in $CHECK_FILE_LIST
do
    if [ -e $INSTALL_DIR/$CHECK_FILE ] && [ -s $INSTALL_DIR/$CHECK_FILE ]; then 
        echo "ok...$CHECK_FILE"
     else
         echo "err...$CHECK_FILE"
         UPDATE_ROLLBACK="true"
    fi
done;

if [ "false" == "$UPDATE_ROLLBACK" ]
then
    echo "    8.1. Install Success. "
    if [ -d /opt/rsupport/rvbox/agent/gQAN5m36 ]; then  # if pacakage version is same
        rm -rf /opt/rsupport/rvbox/agent/gQAN5m36
    fi

    # delete backup conf files.(agent.conf, control.conf, app.sh)
    if [ -e /etc/rsupport/rvbox/agent/agent.confgQAN5m36 ]; then 
       rm /etc/rsupport/rvbox/agent/agent.confgQAN5m36
    fi 
    if [ -e /etc/rsupport/rvbox/agent/control.confgQAN5m36 ]; then 
       rm /etc/rsupport/rvbox/agent/control.confgQAN5m36
    fi 
    if [ -e /etc/init.d/app.shgQAN5m36 ]; then 
       rm /etc/init.d/app.shgQAN5m36
    fi 
    # delete etc files.(liveview_failed.jpg, liveview_rec.jpg)
    if [ -e /opt/rsupport/rvbox/var/tmp/liveview_failed.jpggQAN5m36 ]; then 
       rm /opt/rsupport/rvbox/var/tmp/liveview_failed.jpggQAN5m36
    fi 
    if [ -e /opt/rsupport/rvbox/var/tmp/liveview_rec.jpggQAN5m36 ]; then 
       rm /opt/rsupport/rvbox/var/tmp/liveview_rec.jpggQAN5m36
    fi 

    if [ -L /opt/rsupport/rvbox/agent/latest ]; then
        CURRENT_VERSION_PATH=`readlink /opt/rsupport/rvbox/agent/latest`
        rm /opt/rsupport/rvbox/agent/latest

        if [ "$CURRENT_VERSION_PATH" == "/opt/rsupport/rvbox/agent/1.2.0.01" ]; then   # if pacakage version is same
           echo "pacakage version is same"
        elif [ -d $CURRENT_VERSION_PATH ]; then  # if the package version is different, delete old version
           echo "delete $CURRENT_VERSION_PATH"
           rm -rf $CURRENT_VERSION_PATH
        fi
    fi
    echo "1.2.0.0101" > /opt/rsupport/rvbox/version
    ln -s /opt/rsupport/rvbox/agent/1.2.0.01 /opt/rsupport/rvbox/agent/latest
else
    echo "    8.1. Install Fail.  rollback."
    rm -rf /opt/rsupport/rvbox/agent/1.2.0.01
    if [ -d /opt/rsupport/rvbox/agent/gQAN5m36 ]; then
        mv /opt/rsupport/rvbox/agent/gQAN5m36 /opt/rsupport/rvbox/agent/1.2.0.01
    fi
    # rollback conf files.(agent.conf, control.conf, app.sh)
    if [ -e /etc/rsupport/rvbox/agent/agent.confgQAN5m36 ]; then 
       mv /etc/rsupport/rvbox/agent/agent.confgQAN5m36 /etc/rsupport/rvbox/agent/agent.conf
    fi 

    if [ -e /etc/rsupport/rvbox/agent/control.confgQAN5m36 ]; then 
       mv /etc/rsupport/rvbox/agent/control.confgQAN5m36 /etc/rsupport/rvbox/agent/control.conf
    fi 

    if [ -e /etc/init.d/app.shgQAN5m36 ]; then 
       mv /etc/init.d/app.shgQAN5m36 /etc/init.d/app.sh
    fi 

    # rollback etc files.(liveview_failed.jpg)
    if [ -e /opt/rsupport/rvbox/var/tmp/liveview_failed.jpggQAN5m36 ]; then 
       mv /opt/rsupport/rvbox/var/tmp/liveview_failed.jpggQAN5m36 /opt/rsupport/rvbox/var/tmp/liveview_failed.jpg
    fi 

    if [ -e /opt/rsupport/rvbox/var/tmp/liveview_rec.jpggQAN5m36 ]; then 
       mv /opt/rsupport/rvbox/var/tmp/liveview_rec.jpggQAN5m36 /opt/rsupport/rvbox/var/tmp/liveview_rec.jpg
    fi 

fi

echo "9. Register cron schedule"
echo "* * * * * for i in $(seq -s " " 1 3 60); do /opt/rsupport/rvbox/agent/latest/watchdog.sh agent && sleep 3; done" > /var/spool/cron/crontabs/root
echo "* * * * * for i in $(seq -s " " 1 3 60); do /opt/rsupport/rvbox/web/latest/watchdog.sh web && sleep 3; done" >> /var/spool/cron/crontabs/root

rm /opt/rsupport/rvbox/agent/latest/install-agent.sh

echo "10. Synchronize Storage"
sync

echo "11. Start Watchdog"
/opt/rsupport/rvbox/agent/latest/watchdog.sh agent -d

